/*
 * Copyright (c) 2020 Razeware LLC
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * Notwithstanding the foregoing, you may not use, copy, modify, merge, publish,
 * distribute, sublicense, create a derivative work, and/or sell copies of the
 * Software in any work that is designed, intended, or marketed for pedagogical or
 * instructional purposes related to programming, coding, application development,
 * or information technology.  Permission for such use, copying, modification,
 * merger, publication, distribution, sublicensing, creation of derivative works,
 * or sale is expressly withheld.
 * 
 * This project and source code may use libraries or frameworks that are
 * released under various Open-Source licenses. Use of those libraries and
 * frameworks are governed by their own individual licenses.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package com.raywenderlich.android.instafilter

import android.graphics.ColorMatrix
import android.graphics.ColorMatrixColorFilter
import android.graphics.RenderEffect
import android.graphics.Shader
import android.os.Bundle
import androidx.appcompat.app.AppCompatActivity
import com.raywenderlich.android.instafilter.databinding.ActivityMainBinding

class MainActivity : AppCompatActivity() {

  private lateinit var binding: ActivityMainBinding

  companion object {

    const val DEFAULT_BLUR = 10f
    const val DEFAULT_SATURATION = 10f
    const val DEFAULT_OFFSET = 50f
    const val DEFAULT_FULLSCREEN_BLUR = 30f
  }

  override fun onCreate(savedInstanceState: Bundle?) {
    super.onCreate(savedInstanceState)
    binding = ActivityMainBinding.inflate(layoutInflater)

    binding.blurCheck.setOnCheckedChangeListener { _, isChecked ->
      if (isChecked) {
        binding.saturationCheck.isClickable = false
        binding.offsetEffectsCheck.isClickable = false
        binding.chainEffectCheck.isClickable = false

        binding.blurSlider.isEnabled = true
        val blurEffect = createBlurEffect(DEFAULT_BLUR, DEFAULT_BLUR, Shader.TileMode.MIRROR)
        binding.imageView.setRenderEffect(blurEffect)

        binding.blurSlider.addOnChangeListener { _, value, _ ->
          if (value != 0f) {
            val varyingBlurValue = createBlurEffect(value, value, Shader.TileMode.MIRROR)
            binding.imageView.setRenderEffect(varyingBlurValue)
          }
        }
      } else {
        binding.imageView.setRenderEffect(null)
        binding.blurSlider.isEnabled = false

        binding.saturationCheck.isClickable = true
        binding.offsetEffectsCheck.isClickable = true
        binding.chainEffectCheck.isClickable = true
      }
    }

    binding.saturationCheck.setOnCheckedChangeListener { _, isChecked ->
      if (isChecked) {
        binding.blurCheck.isClickable = false
        binding.offsetEffectsCheck.isClickable = false
        binding.chainEffectCheck.isClickable = false

        binding.colorFilterSlider.isEnabled = true
        val saturation = createSaturationEffect(DEFAULT_SATURATION)
        binding.imageView.setRenderEffect(saturation)

        binding.colorFilterSlider.addOnChangeListener { _, value, _ ->
          val varyingSaturationValue = createSaturationEffect(value)
          binding.imageView.setRenderEffect(varyingSaturationValue)
        }
      } else {
        binding.imageView.setRenderEffect(null)
        binding.colorFilterSlider.isEnabled = false

        binding.blurCheck.isClickable = true
        binding.offsetEffectsCheck.isClickable = true
        binding.chainEffectCheck.isClickable = true
      }
    }

    binding.offsetEffectsCheck.setOnCheckedChangeListener { _, isChecked ->
      if (isChecked) {
        binding.blurCheck.isClickable = false
        binding.saturationCheck.isClickable = false
        binding.chainEffectCheck.isClickable = false

        binding.offsetSlider.isEnabled = true
        val offsetEffect = createOffsetEffect(DEFAULT_OFFSET, DEFAULT_OFFSET)
        binding.imageView.setRenderEffect(offsetEffect)

        binding.offsetSlider.addOnChangeListener { _, value, _ ->
          val varyingOffsetValue = createOffsetEffect(value, value)
          binding.imageView.setRenderEffect(varyingOffsetValue)
        }
      } else {
        binding.imageView.setRenderEffect(null)
        binding.offsetSlider.isEnabled = false

        binding.blurCheck.isClickable = true
        binding.saturationCheck.isClickable = true
        binding.chainEffectCheck.isClickable = true
      }
    }

    binding.chainEffectCheck.setOnCheckedChangeListener { _, isChecked ->
      if (isChecked) {
        binding.blurCheck.isClickable = false
        binding.saturationCheck.isClickable = false
        binding.offsetEffectsCheck.isClickable = false

        binding.chainEffectSlider.isEnabled = true
        applyChainEffect(DEFAULT_BLUR, DEFAULT_SATURATION)

        binding.chainEffectSlider.addOnChangeListener { _, value, _ ->
          applyChainEffect(value, value)
        }
      } else {
        binding.chainEffectSlider.isEnabled = false
        binding.imageView.setRenderEffect(null)

        binding.blurCheck.isClickable = true
        binding.saturationCheck.isClickable = true
        binding.offsetEffectsCheck.isClickable = true
      }
    }

    binding.fullScreenCheck.setOnCheckedChangeListener { _, isChecked ->
      if (isChecked) {
        val blurScreen = createBlurEffect(DEFAULT_FULLSCREEN_BLUR, DEFAULT_FULLSCREEN_BLUR,
            Shader.TileMode.MIRROR)
        binding.root.setRenderEffect(blurScreen)
      } else {
        binding.root.setRenderEffect(null)
      }
    }
    setContentView(binding.root)
  }

  private fun createOffsetEffect(offsetX: Float, offsetY: Float): RenderEffect {
    return RenderEffect.createOffsetEffect(offsetX, offsetY)
  }

  private fun createBlurEffect(radiusX: Float, radiusY: Float, shader: Shader.TileMode):
      RenderEffect {
    return RenderEffect.createBlurEffect(radiusX, radiusY, shader)
  }

  private fun createSaturationEffect(saturation: Float): RenderEffect {
    return RenderEffect.createColorFilterEffect(ColorMatrixColorFilter(
        ColorMatrix().apply {
          setSaturation(saturation)
        }
    ))
  }

  private fun applyChainEffect(blur: Float, saturation: Float) {
    if (blur < 1) {
      return
    } else {
      val blurry = createBlurEffect(blur, blur, Shader.TileMode.MIRROR)
      val saturate = createSaturationEffect(saturation)
      val chainEffect = RenderEffect.createChainEffect(blurry, saturate)
      binding.imageView.setRenderEffect(chainEffect)
    }
  }
}